using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.IO;

using KetLabs.FTP;

namespace FtpClientCS
{
    public partial class FormMain : Form
    {
        private const string MSG_NOT_CONNECTED = "Operation not allowed. You are not connected to a remote server.";

        /// <summary>
        /// Image ID for Folder in the ImageList
        /// </summary>
        private const int FOLDER_IMAGE_INDEX = 0;

        /// <summary>
        /// Image ID for File in the ImageList
        /// </summary>
        private const int FILE_IMAGE_INDEX = 0;

        /// <summary>
        /// FTP client local variable
        /// </summary>
        private FtpClient ftp = new FtpClient();
        
        public FormMain()
        {
            InitializeComponent();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            String about = "KetLabs - Ftp Client Demo\r\n\r\n" +
                "This application demonstrates KetLabs Ftp Library for .NET functionality.\r\n\r\n" +
                "Copyright KetLabs Ltd. 2010 \r\n www.ketlabs.com";
            MessageBox.Show(about, this.Text);
        }

        /// <summary>
        /// Shows status line text
        /// </summary>
        /// <param name="s">message to be shown</param>
        private void ShowMessage(string s)
        {
            lblMessageLine.Text = s;
        }

        /// <summary>
        /// Shows local folder contents
        /// </summary>
        private void RefreshLocalFiles()
        {
            lstFiles.Items.Clear();
            
            if (! txtDir.Text.Equals(Path.GetPathRoot(txtDir.Text)) )
            {
                lstFiles.Items.Add(new ListViewItem(new String[] { "..", "", "" }, 0));
            }
                        
            String[] directories = Directory.GetDirectories(txtDir.Text);
            foreach (String dir in directories)
            {
                ListViewItem item = new ListViewItem(new String[] { Path.GetFileName(dir), "", "" }, 0);
                lstFiles.Items.Add(item);
            }

            String[] files =  Directory.GetFiles(txtDir.Text);
            foreach (String file in files)
            {
                FileInfo f = new FileInfo(file);
                ListViewItem item = new ListViewItem(new String[] { Path.GetFileName(file), 
                                                                    f.Length.ToString(),
                                                                    f.LastAccessTime.ToShortDateString()}, 0);
                item.ImageIndex = 1;
                lstFiles.Items.Add(item);
            }
        }

        /// <summary>
        /// Reloads the remote host file listing if connected.
        /// </summary>
        private void RefreshRemoteFiles()
        {
            if (!ftp.Connected) return;

            FtpFileInfo[] files = ftp.ListCurrentDirectory();

            lstRemoteFiles.Items.Clear();
            foreach (FtpFileInfo file in files)
            {
                if (file.IsDirectory())
                {
                    lstRemoteFiles.Items.Add(new ListViewItem(new String[] { file.Name, "", "" }, 0));
                }
            }
            foreach (FtpFileInfo file in files)
            {
                if (file.IsFile())
                {
                    ListViewItem item = new ListViewItem(new String[] { file.Name, 
                                                                        file.Length.ToString(), 
                                                                        file.Timestamp.ToShortDateString() }, 0);
                    item.ImageIndex = 1;
                    lstRemoteFiles.Items.Add(item);
                }
            }
            txtRemoteDir.Text = ftp.CurrentDirectory;
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            ShowMessage("");

            txtDir.Text = Directory.GetCurrentDirectory();
            txtUrl.Text = "ftp://siteground163.com/public_html/sdk/test";
            txtUsername.Text = "didisoft";
            txtPassword.Text = "silvercolors123";

            RefreshLocalFiles();
        }

        /// <summary>
        /// On Enter key in the txtDir, change directory
        /// </summary>
        private void txtDir_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                RefreshLocalFiles();
            }
        }

        /// <summary>
        /// Changes local folder or uploads a file on double click
        /// </summary>
        private void lstFiles_DoubleClick(object sender, EventArgs e)
        {
            if (lstFiles.SelectedIndices.Count > 0) 
            {
                if (lstFiles.Items[lstFiles.SelectedIndices[0]].ImageIndex == FOLDER_IMAGE_INDEX)
                {
                    // Change local folder
                    string newDir = lstFiles.Items[lstFiles.SelectedIndices[0]].Text;
                    txtDir.Text = Path.Combine(txtDir.Text, newDir);
                    txtDir.Text = Path.GetFullPath(txtDir.Text);
                    RefreshLocalFiles();
                }
                else
                {
                    if (ftp.Connected)
                    {
                        // Upload file
                        string fileForUpload = lstFiles.Items[lstFiles.SelectedIndices[0]].Text;
                        string sourceFile = Path.Combine(txtDir.Text, fileForUpload);
                        string destination = fileForUpload;
                        ftp.Upload(sourceFile, destination);
                    }
                }
            }
        }

        /// <summary>
        /// Changes remote folder or downloads a file on double click 
        /// </summary>
        private void lstRemoteFiles_DoubleClick(object sender, EventArgs e)
        {
            if (!ftp.Connected)
            {
                return;
            } 

            if (lstRemoteFiles.SelectedIndices.Count > 0)
            {
                if (lstRemoteFiles.Items[lstRemoteFiles.SelectedIndices[0]].ImageIndex == FOLDER_IMAGE_INDEX)
                {
                    // Change remote folder
                    ftp.ChangeDirectory(lstRemoteFiles.Items[lstRemoteFiles.SelectedIndices[0]].Text);
                    RefreshRemoteFiles();
                }
                else
                {
                    if (ftp.Connected)
                    {
                        // Download file
                        string sourceFile = lstRemoteFiles.Items[lstRemoteFiles.SelectedIndices[0]].Text;
                        string destinationFile = Path.Combine(txtDir.Text, Path.GetFileName(sourceFile));
                        ftp.Download(sourceFile, destinationFile);
                    }
                }
            }
        }


        /// <summary>
        /// Connects to remote host
        /// </summary>
        private void btnConnect_Click(object sender, EventArgs e)
        {
            // depending on the current connection status change the button label 
            if (!ftp.Connected)
            {
                ftp.Username = txtUsername.Text;
                ftp.Password = txtPassword.Text;
                ftp.Connect(txtUrl.Text);
                RefreshRemoteFiles();
                txtRemoteDir.Text = ftp.CurrentDirectory;
                btnConnect.Text = "Disconnect";
                ShowMessage("Connected to " + ftp.Host);
            }
            else
            {
                ftp.Disconnect();
                lstRemoteFiles.Items.Clear();
                btnConnect.Text = "Connect";
                ShowMessage("");
            }
        }

        /// <summary>
        /// Downloads selected file(s) from remote host
        /// </summary>
        private void btnDownload_Click(object sender, EventArgs e)
        {
            if (!ftp.Connected) return;

            // download eah selected file
            if (lstRemoteFiles.SelectedIndices.Count > 0) {
                for (int i=0; i<lstRemoteFiles.SelectedIndices.Count; i++) {
                    string sourceFile = lstRemoteFiles.Items[lstRemoteFiles.SelectedIndices[i]].Text;
                    string destinationFile = Path.Combine(txtDir.Text, Path.GetFileName(sourceFile));
                    ftp.Download(sourceFile, destinationFile);
                }
            }

            RefreshLocalFiles();
        }

        /// <summary>
        /// Uploads selected file(s) to remote host
        /// </summary>
        private void btnUpload_Click(object sender, EventArgs e)
        {
            if (!ftp.Connected) return;

            // upload each selected file
            if (lstFiles.SelectedIndices.Count > 0)
            {
                for (int i = 0; i < lstFiles.SelectedIndices.Count; i++)
                {
                    string fileForUpload = lstFiles.Items[lstFiles.SelectedIndices[i]].Text;
                    string sourceFile = Path.Combine(txtDir.Text, fileForUpload);
                    string destination = fileForUpload;
                    ftp.Upload(sourceFile, destination);
                }
            }

            RefreshRemoteFiles();
        }

        /// <summary>
        /// Deletes selected file(s) from remote host
        /// </summary>
        private void btnDelete_Click(object sender, EventArgs e)
        {
            if (!ftp.Connected)
            {
                return;
            }

            if (lstRemoteFiles.SelectedIndices.Count > 0)
            {
                // ask for cconfirmation
                string message = "Are you sure you want to delete " +
                                  Convert.ToString(lstRemoteFiles.SelectedIndices.Count) +
                                  " remote files/folders ?";

                if (MessageBox.Show(message, this.Text, MessageBoxButtons.YesNo) == DialogResult.No)
                {
                    return;
                }

                // remove each selected object
                for (int i = 0; i < lstRemoteFiles.SelectedIndices.Count; i++)
                {
                    string sourceFile = lstRemoteFiles.Items[lstRemoteFiles.SelectedIndices[i]].Text;
                    ftp.Delete(sourceFile);
                }
            }

            RefreshRemoteFiles();
        }

        /// <summary>
        /// Handles drag and drop upload from Windows Explorer and Local files list view
        /// </summary>
        private void lstRemoteFiles_DragDrop(object sender, DragEventArgs e)
        {
            if (!ftp.Connected)
            {
                ShowMessage(MSG_NOT_CONNECTED);
                return;
            } 

            // files from Windows Explorer
            if (e.Data.GetDataPresent(DataFormats.FileDrop)) 
            {
                string[] items = (string[])e.Data.GetData(DataFormats.FileDrop);
                // upload each selected file
                foreach (string item in items)
                {
                    FileInfo f = new FileInfo(item);
                    String localFile = f.FullName;
                    ftp.Upload(localFile, f.Name);
                }
            } 
            else if (e.Data.GetDataPresent(typeof(System.String[])))
            {
                // Files from Loal files ListView
                string[] items = (string[])e.Data.GetData(typeof(System.String[]));
                // upload each selected file
                foreach (string item in items)
                {
                    String localFile = Path.Combine(txtDir.Text, item);
                    ftp.Upload(localFile, item);
                }
            }

            RefreshRemoteFiles();
        }

        /// <summary>
        /// Allows drag drop from Windows Explorer
        /// </summary>
        private void lstRemoteFiles_DragEnter(object sender, DragEventArgs e)
        {
            // make sure they're actually dropping files (not text or anything else)
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false) == true ||
                e.Data.GetDataPresent(DataFormats.StringFormat, false) == true)
            {
                // allow them to continue
                // (without this, the cursor stays a "NO" symbol
                e.Effect = DragDropEffects.All;
            }
        }

        private void lstFiles_ItemDrag(object sender, ItemDragEventArgs e)
        {
            List<string> itemIDs = new List<string>((sender as ListView).SelectedItems.Count);
            foreach (ListViewItem item in (sender as ListView).SelectedItems)
            {
                itemIDs.Add(item.Text);
            }
            (sender as ListView).DoDragDrop(itemIDs.ToArray(), DragDropEffects.Move);
        }

        private void lstRemoteFiles_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.Move;
        }

        private void btnRemoteDirUp_Click(object sender, EventArgs e)
        {
            if (!ftp.Connected) return;

            ftp.ChangeDirectoryUp();
            RefreshRemoteFiles();
        }

        /// <summary>
        /// Intended to be called from DlgRenameFile
        /// </summary>
        /// <param name="oldName">remote file or folder to be renamed</param>
        /// <param name="newName">new name of the remote file or folder</param>
        internal void RenameFileHandler(String oldName, String newName)
        {
            ftp.Rename(oldName, newName);
            RefreshRemoteFiles();
        }

        private void btnRename_Click(object sender, EventArgs e)
        {
            if (!ftp.Connected) return;

            if (lstRemoteFiles.SelectedIndices.Count > 0)
            {
                string sourceFile = lstRemoteFiles.Items[lstRemoteFiles.SelectedIndices[0]].Text;
                DlgRenameFile dlgRename = new DlgRenameFile();
                dlgRename.SetParentForm(this);
                dlgRename.SetFileName(sourceFile);
                dlgRename.ShowDialog();
            }
        }

        /// <summary>
        /// Intended to be called from DlgCreateDirectory
        /// </summary>
        /// <param name="directory">remote directory name to be created</param>
        internal void CreateDirectoryHandler(String directory)
        {
            ftp.CreateDirectory(directory);
            RefreshRemoteFiles();
        }

        private void btnCreateDirectory_Click(object sender, EventArgs e)
        {
            if (!ftp.Connected) return;

            DlgCreateDirectory dlgCreate = new DlgCreateDirectory();
            dlgCreate.SetParentForm(this);
            dlgCreate.ShowDialog();
        }
    }
}